import importlib
import sys
from pathlib import Path

import pandas as pd

from domacc.fileReader import DefaultFileReader


class AccountingData:
    def __init__(self, name, path, readerType, initialCash=0, customReaders=None):
        self.name = name
        self.path = path
        self.reader = self._getFileReader(readerType, customReaders)
        self.initialCash = initialCash

    @classmethod
    def LoadFromJsonObject(cls, accDataDict, customReaders):
        name = accDataDict.get("id")
        path = accDataDict.get("bankMovesPath")
        readerType = accDataDict.get("readerType")
        initialCash = accDataDict.get("initialCash")
        return cls(name, path, readerType, initialCash, customReaders)

    @classmethod
    def _getFullPath(cls, path, configFile):
        path = Path(path)
        if not path.is_absolute():
            parent = Path(configFile).parent()
            path = parent / path
        return path

    def _getFileReader(self, readerType, customReaders):
        reader = DefaultFileReader
        if customReaders:
            customReaders = Path(customReaders).absolute()
            sys.path.append(customReaders.parent)
            try:
                readers = importlib.import_module(customReaders.stem)
            except ImportError:
                return DefaultFileReader

            try:
                reader = getattr(readers, readerType)
            except AttributeError:
                return DefaultFileReader
        return reader

    def getMovements(self):
        movements = self.reader.readFolder(self.path)
        if self.initialCash:
            initialCashDF = pd.DataFrame(
                {
                    "Date": [movements.iloc[0]["Date"]],
                    "Concept": ["Initial cash"],
                    "Income": [max(0, self.initialCash)],
                    "Outcome": [max(0, -self.initialCash)],
                }
            )
            movements = pd.concat([initialCashDF, movements], ignore_index=True)

        movements["AccountId"] = self.name
        return movements



import datetime

import pandas as pd


class Budget:
    """Frequencies: daily (D) weekly (W) monthly (M) year (Y)."""

    def __init__(self, accounting, categories, frequency, limit):
        accounting = self.filterAccountingByCategories(accounting, categories)
        self.limit = limit
        self.periodicOutcome = self.getPeriodicOutcome(accounting, frequency)

    @classmethod
    def filterAccountingByCategories(cls, accounting, categories):
        for category in categories:
            mask = accounting["Tags"].str.contains(category)
            accounting = accounting.loc[mask]
        return accounting

    @classmethod
    def getMeanBasedBudget(cls, accounting, categories, frequency):
        accounting = cls.filterAccountingByCategories(accounting, categories)
        periodicOutcome = cls.getPeriodicOutcome(accounting, frequency)
        return cls(accounting, categories, frequency, periodicOutcome.mean())

    @classmethod
    def getPeriodicOutcome(cls, accounting, frequency):
        periodicOutcome = []
        periodRange = pd.period_range(
            start=accounting.iloc[0]["Date"], end=accounting.iloc[-1]["Date"], freq=frequency
        )
        for period in periodRange:
            mask1 = accounting["Date"] >= period.start_time
            mask2 = accounting["Date"] <= period.end_time
            df = accounting.loc[mask1 & mask2]
            periodicOutcome.append(sum(df["Outcome"]))
        return pd.DataFrame(periodicOutcome, index=periodRange)

    def getPeriodicRemaining(self):
        return self.limit - self.periodicOutcome

    def getCurrentRemaining(self):
        remaining = self.getPeriodicRemaining()
        now = datetime.datetime.now()
        for date, row in remaining.iterrows():
            if date.start_time <= now <= date.end_time:
                return row[0]

        return self.limit



from os import listdir
from os.path import isfile, join

import pandas as pd


class FileReader:
    @classmethod
    def readFolder(cls, path):
        dfs = []
        for filename in listdir(path):
            if isfile(join(path, filename)) and filename.endswith(cls.fileExt):
                df = cls.readFile(join(path, filename))
                dfs.append(df)

        return pd.concat(dfs, ignore_index=True)

    @classmethod
    def readFile(cls, filename):
        """Implement in subclass."""
        raise NotImplementedError

    @classmethod
    def isIncome(cls, row):
        if row[cls.amountColIndex] >= 0:
            return row[cls.amountColIndex]
        return 0

    @classmethod
    def isOutcome(cls, row):
        if row[cls.amountColIndex] < 0:
            return -row[cls.amountColIndex]
        return 0


class DefaultFileReader(FileReader):
    fileExt = ".csv"
    amountColIndex = 4

    @classmethod
    def readFile(cls, filename):
        df = pd.read_csv(filename)
        df["Income"] = df.apply(cls.isIncome, axis=1)
        df["Outcome"] = df.apply(cls.isOutcome, axis=1)
        df = df.drop(df.columns[[2]], axis=1)
        df.columns = ["Date", "Concept", "Income", "Outcome"]
        df["Date"] = pd.to_datetime(df["Date"], format="%d/%m/%Y")  # noqa: WPS323
        return df.sort_values(by="Date")



from setuptools import setup

setup(
    name="domacc",
    version="0.1",
    description="Domestic accounting python package",
    url="http://gitlab.com/pepe.bari/domacc",
    author="Pepe Bari",
    author_email="pepe.bari@gmail.com",
    license="MIT",
    packages=["domacc"],
    install_requires=["pandas"],
    zip_safe=False,
    test_suite='nose.collector',
    tests_require=['nose'],
)



import json
import re

import pandas as pd

from domacc.accountingData import AccountingData


class Accounting:
    def __init__(self, accountingData, categories):
        self.accountingData = accountingData
        self.categories = categories

    def getAccounting(self, initialDate=None, finalDate=None):
        movements = self.getMovements(initialDate, finalDate)
        movements = self.addBalance(movements)
        return self.addCategories(movements)

    @classmethod
    def LoadFromJsonFile(cls, configFile):
        with open(configFile) as jsonFile:
            jsonData = json.load(jsonFile)
            accountingData = cls._getAccountingData(jsonData)
            categories = jsonData.get("categories", {})
        return cls(accountingData, categories)

    @classmethod
    def _getAccountingData(cls, jsonData):
        accountingData = []
        customReaders = jsonData.get("customReaders")
        for accDataDict in jsonData["data"]:
            accountingData.append(
                AccountingData.LoadFromJsonObject(accDataDict, customReaders)
            )
        return accountingData

    def getMovements(self, initialDate, finalDate):
        dfs = []
        for accData in self.accountingData:
            dfs.append(accData.getMovements())

        df = pd.concat(dfs, ignore_index=True)
        df = df.sort_values(by="Date")
        if initialDate:
            mask = df["Date"] >= initialDate
            df = df.loc[mask]
        if finalDate:
            mask = df["Date"] <= finalDate
            df = df.loc[mask]
        df.index = range(len(df))
        return df

    def addBalance(self, movements):
        movements["Balance"] = 0
        for index, _ in movements.iterrows():
            previousBalance = movements.loc[index - 1, "Balance"] if index else 0
            income = movements.loc[index, "Income"]
            outcome = movements.loc[index, "Outcome"]
            movements.loc[index, "Balance"] = previousBalance + income - outcome
        return movements

    def addCategories(self, movements):
        movements["Tags"] = ""
        for index, row in movements.iterrows():
            newTags = []
            for category in self.categories:
                if re.search(category["regex"], row["Concept"]):
                    newTags.extend(category["tags"])
            movements.at[index, "Tags"] = ",".join(set(newTags))
        return movements
